==> TuxGuitar-0.7-src.tar.gz: TuxGuitar-0.7-src/src/org/herac/tuxguitar/song/models/Duration.java <==
/*
 * Created on 29-nov-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.herac.tuxguitar.song.models;

import java.io.Serializable;
/**
 * @author julian
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Duration implements Serializable{    
    /**
     * tiempo por defecto de la Negra.
     */
    public static final long QUARTER_TIME = 1000;    
    /**
     * Redonda
     */
    public static final int WHOLE = 1;
    
    /**
     * Blanca.
     */
    public static final int HALF = 2;
    
    /**
     * Negra.
     */
    public static final int QUARTER = 4;
    
    /**
     * Corchea.
     */
    public static final int EIGHTH = 8;
    
    /**
     * Semi-Corchea.
     */
    public static final int SIXTEENTH = 16;
    
    /**
     * Fusa.
     */
    public static final int THIRTY_SECOND = 32;
    
    /**
     * Semi-Fusa.
     */
    public static final int SIXTY_FOURTH = 64;    
           
    /**
     * Sin tupleto
     */    
    public static final Tupleto NO_TUPLETO = new Tupleto(1,1);
    /**
     * Valor
     */
    private int value;
    /**
     * Puntillo
     */
    private boolean dotted;
    /**
     * Doble Puntillo
     */
    private boolean doubleDotted;    
    /**
     * Tupleto
     */
    private Tupleto tupleto;
    
    public Duration(int value,boolean dotted,boolean doubleDotted,Tupleto tupleto){
        this.value = value;    
        this.dotted = dotted;
        this.doubleDotted = doubleDotted;
        this.tupleto = tupleto;
    }
    
    public Duration(int value){
        this(value,false,false,new Tupleto(1,1));
    }
    
    public int getValue() {
        return value;
    }
    public void setValue(int value) {
        this.value = value;
    }
    
    public boolean isDotted() {
        return dotted;
    }
    public void setDotted(boolean dotted) {
        this.dotted = dotted;
    }
            
    public boolean isDoubleDotted() {
        return doubleDotted;
    }
    
    public void setDoubleDotted(boolean doubleDotted) {
        this.doubleDotted = doubleDotted;
    }
    
    public Tupleto getTupleto(){
        return this.tupleto;
    }
    
    public long getTime(){
        long time = 0;
        switch(this.value){
        	case WHOLE:
        	    time = QUARTER_TIME * 4;
        	    break;
        	case HALF:
        	    time = QUARTER_TIME * 2;
        	    break;        	            	    
        	case QUARTER:
        	    time = QUARTER_TIME;
        	    break;        	            	    
        	case EIGHTH:
        	    time = QUARTER_TIME / 2;
        	    break;        	            	    
        	case SIXTEENTH:
        	    time = QUARTER_TIME / 4;
        	    break;        	            	    
        	case THIRTY_SECOND:
        	    time = QUARTER_TIME / 8;
        	    break;        	            	    
        	case SIXTY_FOURTH:
        	    time = QUARTER_TIME / 16;
        	    break;        	            	    
        }
        if(this.dotted){
            time += time / 2;
        }else if(this.doubleDotted){
            time += ((time / 4) * 3);
        }
        
        return this.tupleto.convertTime(time);
    }
    
    public static Duration fromTime(long time){
        Duration duration = new Duration(Duration.WHOLE);
        duration.setDotted(true);
    	
        boolean finish = false;
        while(!finish){
            long currentDurationTime = duration.getTime();
            if(currentDurationTime <= time){
                return duration;
            }else{
                if(duration.isDotted()){                
                	duration.setDotted(false);
                }else{
                	duration.setValue(duration.getValue() * 2);
                	duration.setDotted(true);
                }
            }
            if(duration.getValue() > Duration.SIXTY_FOURTH){
                finish = true;
            }
        }
        return null;
    }
    
    public Object clone(){
        return new Duration(this.value,this.dotted,this.doubleDotted,(Tupleto)this.tupleto.clone());
    }
}

==> TuxGuitar-0.8-src.tar.gz: TuxGuitar-0.8-src/src/org/herac/tuxguitar/song/models/Duration.java <==
/*
 * Created on 29-nov-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package org.herac.tuxguitar.song.models;

import java.io.Serializable;
/**
 * @author julian
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Duration implements Serializable{    
    /**
     * tiempo por defecto de la Negra.
     */
    public static final long QUARTER_TIME = 1000;    
    /**
     * Redonda
     */
    public static final int WHOLE = 1;
    
    /**
     * Blanca.
     */
    public static final int HALF = 2;
    
    /**
     * Negra.
     */
    public static final int QUARTER = 4;
    
    /**
     * Corchea.
     */
    public static final int EIGHTH = 8;
    
    /**
     * Semi-Corchea.
     */
    public static final int SIXTEENTH = 16;
    
    /**
     * Fusa.
     */
    public static final int THIRTY_SECOND = 32;
    
    /**
     * Semi-Fusa.
     */
    public static final int SIXTY_FOURTH = 64;    
           
    /**
     * Sin tupleto
     */    
    public static final Tupleto NO_TUPLETO = new Tupleto(1,1);
    /**
     * Valor
     */
    private int value;
    /**
     * Puntillo
     */
    private boolean dotted;
    /**
     * Doble Puntillo
     */
    private boolean doubleDotted;    
    /**
     * Tupleto
     */
    private Tupleto tupleto;
    
    public Duration(int value,boolean dotted,boolean doubleDotted,Tupleto tupleto){
        this.value = value;    
        this.dotted = dotted;
        this.doubleDotted = doubleDotted;
        this.tupleto = tupleto;
    }
    
    public Duration(int value){
        this(value,false,false,new Tupleto(1,1));
    }
    
    public int getValue() {
        return value;
    }
    public void setValue(int value) {
        this.value = value;
    }
    
    public boolean isDotted() {
        return dotted;
    }
    public void setDotted(boolean dotted) {
        this.dotted = dotted;
    }
            
    public boolean isDoubleDotted() {
        return doubleDotted;
    }
    
    public void setDoubleDotted(boolean doubleDotted) {
        this.doubleDotted = doubleDotted;
    }
    
    public Tupleto getTupleto(){
        return this.tupleto;
    }
    
    public long getTime(){
        long time = 0;
        switch(this.value){
        	case WHOLE:
        	    time = QUARTER_TIME * 4;
        	    break;
        	case HALF:
        	    time = QUARTER_TIME * 2;
        	    break;        	            	    
        	case QUARTER:
        	    time = QUARTER_TIME;
        	    break;        	            	    
        	case EIGHTH:
        	    time = QUARTER_TIME / 2;
        	    break;        	            	    
        	case SIXTEENTH:
        	    time = QUARTER_TIME / 4;
        	    break;        	            	    
        	case THIRTY_SECOND:
        	    time = QUARTER_TIME / 8;
        	    break;        	            	    
        	case SIXTY_FOURTH:
        	    time = QUARTER_TIME / 16;
        	    break;        	            	    
        }
        if(this.dotted){
            time += time / 2;
        }else if(this.doubleDotted){
            time += ((time / 4) * 3);
        }
        
        return this.tupleto.convertTime(time);
    }
    
    public static Duration fromTime(long time){
    	return fromTime(time,new Duration(Duration.SIXTY_FOURTH,false,false,new Tupleto(3,2)));
    }
    
    public static Duration fromTime(long time,Duration minDuration){
    	//Duration duration =  new Duration(Duration.SIXTY_FOURTH,false,false,new Tupleto(3,2));
    	Duration duration = (Duration)minDuration.clone();
    	
    	int marginTime = 10;
        
    	Duration tempDuration = new Duration(Duration.WHOLE);
        tempDuration.setDotted(true);
    	
        boolean finish = false;        
        while(!finish){
            long currentDurationTime = tempDuration.getTime();
            if(currentDurationTime - marginTime <= time){
            	if(currentDurationTime > duration.getTime()){
            		duration = (Duration)tempDuration.clone();
            	}
             
            }
            if(tempDuration.isDotted()){                
               	tempDuration.setDotted(false);
            }else if(tempDuration.getTupleto().isEqual(Duration.NO_TUPLETO)){
               	tempDuration.getTupleto().setEnters(3);
               	tempDuration.getTupleto().setTimes(2);
            }else{
               	tempDuration.setValue(tempDuration.getValue() * 2);
               	tempDuration.setDotted(true);            
               	tempDuration.getTupleto().setEnters(1);
               	tempDuration.getTupleto().setTimes(1);
            }
            if(tempDuration.getValue() > Duration.SIXTY_FOURTH){
                finish = true;
            }
        }        
        return duration;
        
        
        /*
        while(!finish){
            long currentDurationTime = duration.getTime();
            if(currentDurationTime - marginTime <= time){
                return duration;
            }else{
                if(duration.isDotted()){                
                	duration.setDotted(false);
                }else{
                	duration.setValue(duration.getValue() * 2);
                	duration.setDotted(true);                	
                }
            }
            if(duration.getValue() > Duration.SIXTY_FOURTH){
                finish = true;
            }
        }
        return null;
        */
    }
    
    public boolean isEqual(Duration d){
    	return (getValue() == d.getValue() && isDotted() == d.isDotted() && isDoubleDotted() == d.isDoubleDotted() && getTupleto().isEqual(d.getTupleto()));
    }
    
    public Object clone(){
        return new Duration(this.value,this.dotted,this.doubleDotted,(Tupleto)this.tupleto.clone());
    }
}
